## Python 3.11.7
import pandas as pd # Version: 2.1.4
import numpy as np # Version: 1.26.4
import matplotlib.pyplot as plt #Version: 3.8.0
import seaborn as sns # Version: 0.12.2
# sklearn Version: 0.12.2
from sklearn.tree import DecisionTreeClassifier
from sklearn.model_selection import StratifiedKFold, train_test_split, cross_val_score
from sklearn.metrics import roc_auc_score

## URL of dataset: https://www.kaggle.com/datasets/uciml/pima-indians-diabetes-database

col_names = ['pregnant', 'glucose', 'bp', 'skin', 'insulin', 'bmi', 'pedigree', 'age', 'label']
pima = pd.read_csv(r"path", header=None, names=col_names).loc[1:]
pima.label = pima.label.astype(int) 

sample_sizes = [150, 400, len(pima)]
num_repeats_list = [1, 20]

clf = DecisionTreeClassifier(random_state=1)

# Create a grid of subplots: 4 rows, 3 columns
fig, axes = plt.subplots(nrows=4, ncols=3, figsize=(18, 24))
plt.subplots_adjust(hspace=0.3, wspace=0.1)  # Adjust spacing between plots

# Set column headers for sample sizes
for j, sample_size in enumerate(sample_sizes):
    axes[0, j].set_title(f'Sample size = {sample_size}', fontsize=16, pad=20)

## Add formatter for subplot on main plot   
col_pos = 0 
## Set font size
fs = 16 

for i, sample_size in enumerate(sample_sizes):
   
    ## Select sample whilst maintaining proportions classes
    pima_sample = pd.concat([pima[pima.label == 1].sample(round(sample_size * 0.349), random_state=1),
                             pima[pima.label == 0].sample(round(sample_size * 0.651), random_state=1)], axis=0)
    
       
    ## Define X and y variables
    feature_cols = ['pregnant', 'insulin', 'bmi', 'age', 'glucose', 'bp', 'pedigree']
    X = pima_sample[feature_cols]
    y = pima_sample.label

    # Add formatter for subplot on main plot     
    row_pos = 0 # Row position of subplot on main plot

    for j, num_repeats in enumerate(num_repeats_list):
        ymax = 2.5 if num_repeats == 1 else 15

        # Train-test split ROC AUC scores
        roc_auc_scores = []
        for rs in range(num_repeats):
            X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.2, random_state= rs,
                                                                stratify=y)
            clf_fit = clf.fit(X_train, y_train)
            score = roc_auc_score(y_test, clf_fit.predict(X_test))
            roc_auc_scores.append(score)

        # Plot histogram for train-test split
        ax = axes[row_pos, col_pos]
        row_pos = row_pos + 1
        sns.histplot(roc_auc_scores, kde=False, binwidth=0.01, ax=ax)
        if num_repeats == 20:
            sns.kdeplot(roc_auc_scores, color='red', ax=ax)
        ax.set_title('Train-test split', fontsize=fs+1)
        ax.set_xlabel('AUC-ROC Score', fontsize=fs-1)
        ax.set_ylabel('Count', fontsize=fs)
        ax.set_xlim(0.4, 1)
        ax.set_ylim(0, ymax)
        ax.tick_params(axis='both', labelsize=fs-1)
        ax.text(0.95, 0.95, f'Sample size: {sample_size}\nRepeats: {num_repeats}',
                ha='right', va='top', fontsize=fs, transform=ax.transAxes,
                bbox=dict(boxstyle='round,pad=0.3', edgecolor='black', facecolor='whitesmoke', alpha=0.7))


        # Cross-validation ROC AUC scores
        roc_auc_scores = []
        for rs in range(num_repeats):
            scores = cross_val_score(clf, X, y, cv=StratifiedKFold(5, random_state=rs, shuffle=True),
                                     scoring='roc_auc')
            roc_auc_scores.extend(scores)

        # Plot histogram for cross-validation
        ax = axes[row_pos, col_pos]
        row_pos = row_pos + 1
        sns.histplot(roc_auc_scores, kde=False, binwidth=0.01, ax=ax)
        if num_repeats == 20:
            sns.kdeplot(roc_auc_scores, color='red', ax=ax)
        ax.set_title('5-Fold Cross-validation', fontsize=fs+1)
        ax.set_xlabel('AUC-ROC Score', fontsize=fs-1)
        ax.set_ylabel('Count', fontsize=fs)
        ax.set_xlim(0.4, 1)
        ax.set_ylim(0, ymax)
        ax.tick_params(axis='both', labelsize=fs-1)  
        ax.text(0.95, 0.95, f'Sample size: {sample_size}\nRepeats: {num_repeats}',
                ha='right', va='top', fontsize=fs, transform=ax.transAxes, 
                bbox=dict(boxstyle='round,pad=0.3', edgecolor='black', facecolor='whitesmoke', alpha=0.7))


    col_pos = col_pos + 1 

# Remove y-ticks and y-labels from subplots that are not in the first column
for i in range(4):
    for j in range(1, 3):  # Start from column 1 to 2 (excluding column 0)
        ax = axes[i, j]
        ax.set_yticks([])
        ax.set_ylabel('')
## Save plot
plt.savefig(r"path", dpi=300, bbox_inches='tight')  
plt.show()


## Number of possible combinations means repeated test sets are highly unlikely.
## In smaller datasets, however, this should be accounted for.
            

    
    
    
    
    